@@ -33,6 +33,7 @@ def flow_to_json(flow: mitmproxy.flow.Flow) -> dict:
     f = {
         "id": flow.id,
         "intercepted": flow.intercepted,
+        "is_replay": flow.is_replay,
         "client_conn": flow.client_conn.get_state(),
         "server_conn": flow.server_conn.get_state(),
         "type": flow.type,
@@ -72,7 +73,7 @@ def flow_to_json(flow: mitmproxy.flow.Flow) -> dict:
                 "contentHash": content_hash,
                 "timestamp_start": flow.request.timestamp_start,
                 "timestamp_end": flow.request.timestamp_end,
-                "is_replay": flow.request.is_replay,
+                "is_replay": flow.is_replay == "request",  # TODO: remove, use flow.is_replay instead.
                 "pretty_host": flow.request.pretty_host,
             }
         if flow.response:
@@ -91,8 +92,11 @@ def flow_to_json(flow: mitmproxy.flow.Flow) -> dict:
                 "contentHash": content_hash,
                 "timestamp_start": flow.response.timestamp_start,
                 "timestamp_end": flow.response.timestamp_end,
-                "is_replay": flow.response.is_replay,
+                "is_replay": flow.is_replay == "response",  # TODO: remove, use flow.is_replay instead.
             }
+            if flow.response.data.trailers:
+                f["response"]["trailers"] = tuple(flow.response.data.trailers.items(True))
+
     f.get("server_conn", {}).pop("cert", None)
     f.get("client_conn", {}).pop("mitmcert", None)
 
@@ -120,7 +124,7 @@ class RequestHandler(tornado.web.RequestHandler):
         if isinstance(chunk, list):
             chunk = tornado.escape.json_encode(chunk)
             self.set_header("Content-Type", "application/json; charset=UTF-8")
-        super(RequestHandler, self).write(chunk)
+        super().write(chunk)
 
     def set_default_headers(self):
         super().set_default_headers()
@@ -301,10 +305,14 @@ class FlowHandler(RequestHandler):
                             request.headers.clear()
                             for header in v:
                                 request.headers.add(*header)
+                        elif k == "trailers":
+                            request.trailers.clear()
+                            for trailer in v:
+                                request.trailers.add(*trailer)
                         elif k == "content":
                             request.text = v
                         else:
-                            raise APIError(400, "Unknown update request.{}: {}".format(k, v))
+                            raise APIError(400, f"Unknown update request.{k}: {v}")
 
                 elif a == "response" and hasattr(flow, "response"):
                     response = flow.response
@@ -317,12 +325,16 @@ class FlowHandler(RequestHandler):
                             response.headers.clear()
                             for header in v:
                                 response.headers.add(*header)
+                        elif k == "trailers":
+                            response.trailers.clear()
+                            for trailer in v:
+                                response.trailers.add(*trailer)
                         elif k == "content":
                             response.text = v
                         else:
-                            raise APIError(400, "Unknown update response.{}: {}".format(k, v))
+                            raise APIError(400, f"Unknown update response.{k}: {v}")
                 else:
-                    raise APIError(400, "Unknown update {}: {}".format(a, b))
+                    raise APIError(400, f"Unknown update {a}: {b}")
         except APIError:
             flow.revert()
             raise
@@ -383,7 +395,7 @@ class FlowContent(RequestHandler):
             filename = self.flow.request.path.split("?")[0].split("/")[-1]
 
         filename = re.sub(r'[^-\w" .()]', "", filename)
-        cd = "attachment; filename={}".format(filename)
+        cd = f"attachment; filename={filename}"
         self.set_header("Content-Disposition", cd)
         self.set_header("Content-Type", "application/text")
         self.set_header("X-Content-Type-Options", "nosniff")
@@ -437,14 +449,14 @@ class Settings(RequestHandler):
 
     def put(self):
         update = self.json
-        option_whitelist = {
+        allowed_options = {
             "intercept", "showhost", "upstream_cert", "ssl_insecure",
             "rawtcp", "http2", "websocket", "anticache", "anticomp",
             "stickycookie", "stickyauth", "stream_large_bodies"
         }
         for k in update:
-            if k not in option_whitelist:
-                raise APIError(400, "Unknown setting {}".format(k))
+            if k not in allowed_options:
+                raise APIError(400, f"Unknown setting {k}")
         self.master.options.update(**update)
 
 
@@ -457,7 +469,7 @@ class Options(RequestHandler):
         try:
             self.master.options.update(**update)
         except Exception as err:
-            raise APIError(400, "{}".format(err))
+            raise APIError(400, f"{err}")
 
 
 class SaveOptions(RequestHandler):
@@ -496,7 +508,7 @@ class Application(tornado.web.Application):
         self.add_handlers("dns-rebind-protection", [(r"/.*", DnsRebind)])
         self.add_handlers(
             # make mitmweb accessible by IP only to prevent DNS rebinding.
-            r'^(localhost|[0-9.:\[\]]+)$',
+            r'^(localhost|[0-9.]+|\[[0-9a-fA-F:]+\])$',
             [
                 (r"/", IndexHandler),
                 (r"/filter-help(?:\.json)?", FilterHelp),
